import { createSignal } from 'solid-js';
import { gql } from '@urql/core';
import { createMutation } from '@urql/solid';

const LOGIN_MUTATION = gql`
  mutation Login($email: String!, $password: String!) {
    login(email: $email, password: $password) {
      accessToken
      refreshToken
      user {
        id
        username
        email
      }
    }
  }
`;

interface LoginFormProps {
  onLoginSuccess: (token: string, userId: string) => void;
}

export function LoginForm(props: LoginFormProps) {
  const [email, setEmail] = createSignal('');
  const [password, setPassword] = createSignal('');
  const [error, setError] = createSignal('');

  const [loginState, login] = createMutation(LOGIN_MUTATION);

  const handleSubmit = async (e: Event) => {
    e.preventDefault();

    if (!email() || !password()) {
      setError('Please enter both email and password');
      return;
    }

    try {
      const result = await login({
        email: email(),
        password: password(),
      });

      if (result.error) {
        setError(result.error.message);
        return;
      }

      if (result.data?.login) {
        const { accessToken, user } = result.data.login;
        localStorage.setItem('token', accessToken);
        localStorage.setItem('userId', user.id);
        props.onLoginSuccess(accessToken, user.id);
      } else {
        setError('Login failed: No data received from server');
      }
    } catch (err) {
      setError(err instanceof Error ? err.message : 'An error occurred');
    }
  };

  return (
    <div class='login-form'>
      <h2>Login</h2>
      {error() && <div class='error'>{error()}</div>}
      <form onSubmit={handleSubmit}>
        <div class='form-group'>
          <label for='email'>Email</label>
          <input
            type='email'
            id='email'
            value={email()}
            onInput={(e) => setEmail(e.currentTarget.value)}
            required
          />
        </div>
        <div class='form-group'>
          <label for='password'>Password</label>
          <input
            type='password'
            id='password'
            value={password()}
            onInput={(e) => setPassword(e.currentTarget.value)}
            required
          />
        </div>
        <div>
          <button type='submit' disabled={loginState.fetching}>
            {loginState.fetching ? 'Logging in...' : 'Login'}
          </button>
        </div>
      </form>
    </div>
  );
}
